<?php

namespace Drupal\entity_template_ui\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;
use Drupal\typed_data\DataFetcherInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class AutocompleteController extends ControllerBase {

  /**
   * The data fetcher.
   *
   * @var \Drupal\typed_data\DataFetcherInterface
   */
  protected $dataFetcher;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('typed_data.data_fetcher')
    );
  }

  /**
   * AutocompleteController constructor.
   *
   * @param \Drupal\typed_data\DataFetcherInterface $data_fetcher
   */
  public function __construct(DataFetcherInterface $data_fetcher) {
    $this->dataFetcher = $data_fetcher;
  }

  /**
   * Autocomplete for data selection.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *
   * @todo: Work out how to filter by permitted data types.
   */
  public function autocomplete(Request $request, BlueprintStorageInterface $blueprint_storage, $template_key, $component_id, $sub_template = NULL) {
    $template = $blueprint_storage->getTemplate($template_key);
    if (is_callable([$template, 'getContextDefinitions'])) {
      $context_definitions = $template->getContextDefinitions();
    }

    if ($sub_template) {
      $path = explode(':', $sub_template);
      while ($bit = array_shift($path)) {
        /** @var \Drupal\entity_template\Plugin\EntityTemplate\Template\SubTemplateInterface $template */
        $template = $template->getComponent($bit);

        if (!$template || !($template instanceof TemplateInterface)) {
          throw new NotFoundHttpException();
        }
      }

      $context_definitions = $template->getContextProvidingTemplate()->getContextDefinitions();
    }
    $component = $template->getComponent($component_id);

    $definitions = [];
    foreach ($context_definitions as $name => $definition) {
      $definitions[$name] = $definition->getDataDefinition();
    }

    $results = $this->dataFetcher->autocompletePropertyPath($definitions, $request->query->get('q'));
    return new JsonResponse($results);

  }

}
