<?php

namespace Drupal\entity_template_ui\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_template\Entity\TemplateBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

class TemplateBuilderDefaultBlueprintController extends ControllerBase {

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity.form_builder')
    );
  }

  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    EntityFormBuilderInterface $entity_form_builder
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFormBuilder = $entity_form_builder;
  }

  /**
   * @param \Drupal\entity_template\Entity\TemplateBuilder $entity_template_builder
   */
  public function editDefaultBlueprint(TemplateBuilder $entity_template_builder) {
    $blueprint_storage = $this->entityTypeManager->getStorage('entity_template_blueprint');

    if (!$entity_template_builder->get('default_blueprint')) {
      $entity_template_builder->set('default_blueprint', $entity_template_builder->id().'.default_blueprint');
      $entity_template_builder->save();
    }

    if (!($blueprint = $blueprint_storage->load($entity_template_builder->get('default_blueprint')))) {
      $blueprint = $blueprint_storage->create([
        'id' => $entity_template_builder->get('default_blueprint'),
        'label' => $entity_template_builder->label().' Default',
        'builder' => "config:".$entity_template_builder->id(),
        'is_default' => TRUE,
      ]);
    }

    $operation = "default";
    if ($this->entityTypeManager->getDefinition('entity_template_blueprint')->getFormClass('edit')) {
      $operation = 'edit';
    }
    return $this->entityFormBuilder->getForm($blueprint, $operation);
  }

  /**
   * @param \Drupal\entity_template\Entity\TemplateBuilder $entity_template_builder
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  public function editDefaultBlueprintTitle(TemplateBuilder $entity_template_builder) {
    return $this->t('Edit @label Default Blueprint', ['@label' => $entity_template_builder->label()]);
  }
}
