<?php

namespace Drupal\entity_template_ui\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Ajax\AjaxFormHelperTrait;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormFactoryInterface;
use Drupal\Core\Plugin\PluginWithFormsInterface;
use Drupal\Core\Url;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\BlueprintTempstoreRepository;
use Drupal\entity_template\TemplateComponentManager;
use Drupal\entity_template\TemplateManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class AddComponentForm extends ComponentFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'add_component_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    $op = "add",
    BlueprintStorageInterface $blueprint_storage = NULL,
    $template_id = '',
    $sub_template = '',
    $plugin_id = ''
  ) {
    $this->initializeTemplate($blueprint_storage, $template_id, $sub_template);

    if (!$this->plugin) {
      $this->plugin = $this->componentManager->createInstance($plugin_id);
    }

    return parent::doBuildForm($form, $form_state, $op);
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state, string $op = "add") {
    parent::submitForm($form, $form_state, $op);

    $this->template->addComponent($this->plugin->getConfiguration());
    $this->blueprintTempstore->set($this->blueprintStorage);

    $form_state->setRedirectUrl(
      $this->blueprintStorage->getEditTemplateUrl($this->rootTemplate->getKey())
    );
  }


}
