<?php

namespace Drupal\entity_template_ui\PluginForm;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\PluginFormBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\entity_template\TemplateComponentManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TemplateConfigureForm
 *
 * @package Drupal\entity_template_ui\PluginForm
 */
class TemplateConfigureForm extends TemplateConfigureFormBase {

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\ConditionalTemplateInterface
   */
  protected $plugin;

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $config = $this->plugin->getConfiguration();

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => new TranslatableMarkup('Label'),
      '#description' => new TranslatableMarkup(
        'A label to help administrators view this template.'
      ),
      '#default_value' => $this->plugin->label(),
      '#weight' => -10,
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#description' => $this->t('A short description of this template. This is only visible to administrators.'),
      '#size' => 3,
      '#default_value' => isset($config['description']) ? $config['description'] : '',
      '#weight' => -5,
    ];

    // Conditions
    $form['conditions'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    if ($url = $this->getTemplateUi()->addConditionUrl()) {
      $form['conditions']['__add'] = [
        '#type' => 'link',
        '#title' => $this->t('Add Condition'),
        '#url' => $url->mergeOptions([
            'attributes' => $this->ajaxAttributes(),
          ]),
        '#weight' => -5,
        '#attributes' => [
          'class' => ['add-condition-button', 'btn', 'button'],
        ],
      ];
    }
    $form['conditions']['table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Condition'),
        $this->t('Operations'),
      ],
      '#empty' => $this->t('This template will always apply in this blueprint.'),
    ];
    foreach ($this->plugin->getConditions() as $k => $condition) {
      /** @var \Drupal\Core\Condition\ConditionInterface $condition */
      $form['conditions']['table'][$k] = [
        'summary' => [
          '#markup' => $condition->summary(),
        ],
        'operations' => [
          '#type' => 'dropbutton',
          '#links' => [
            'configure' => [
              'title' => $this->t('Configure'),
              'url' => $this->getTemplateUi()
                ->configureConditionUrl($k)
                ->mergeOptions([
                  'attributes' => $this->ajaxAttributes(),
                ]),
            ],
            'remove' => [
              'title' => $this->t('Remove'),
              'url' => $this->getTemplateUi()
                ->removeConditionUrl($k)
                ->mergeOptions([
                  'attributes' => $this->ajaxAttributes(),
                ]),
            ],
          ]
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $configuration = $this->plugin->getConfiguration();
    $configuration['label'] = $form_state->getValue('label');
    $configuration['description'] = $form_state->getValue('description');
    $configuration['conditions'] = $this->plugin->getConditions()->getConfiguration();
    $this->plugin->setConfiguration($configuration);
  }

}
