<?php

namespace Drupal\entity_template;

use Drupal\Core\Entity\EntityInterface;

/**
 * Class BlueprintResult
 *
 * @package Drupal\entity_template
 */
class BlueprintResult {

  /**
   * The generated entites keyed by template.
   *
   * @var \Drupal\Core\Entity\EntityInterface[]
   */
  protected $items;

  /**
   * The number of times each template was executed.
   *
   * @var integer[][]
   */
  protected $counter;

  /**
   * Messages set by each template.
   *
   * @var string[][]|\Drupal\Component\Render\MarkupInterface[][]
   */
  protected $messages;

  /**
   * Add an item to the result.
   *
   * @param string $key
   * @param \Drupal\Core\Entity\EntityInterface $entity
   */
  public function addItem($key, EntityInterface $entity) {
    $this->items[$key] = $entity;
  }

  /**
   * Find out whether this blueprint result has a given item.
   *
   * @param string $key
   *
   * @return bool
   */
  public function hasItem($key) {
    return !empty($this->items[$key]);
  }

  /**
   * Get the generated items.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   */
  public function getItems() {
    return $this->items;
  }

  /**
   * Set the items/
   *
   * @param \Drupal\Core\Entity\EntityInterface[] $items
   *
   * @return static
   */
  public function setItems($items) {
    $this->items = $items;

    return $this;
  }

  /**
   * Add a message to the result.
   *
   * @param $key
   * @param $message
   */
  public function addMessage($key, $message) {
    if (!isset($this->messages[$key])) {
      $this->messages[$key] = [];
    }

    $this->messages[$key][] = $message;

    return $this;
  }

  /**
   * Return the messages for a given template key.
   *
   * @param $key
   *
   * @return array|\Drupal\Component\Render\MarkupInterface[]|string[]
   */
  public function getMessages($key) {
    return isset($this->messages[$key]) ? $this->messages[$key] : [];
  }

  /**
   * Get all the messages.
   *
   * @return \Drupal\Component\Render\MarkupInterface[][]|\string[][]
   */
  public function getAllMessages() {
    return $this->messages;
  }

  /**
   * Log that a template has been executed.
   *
   * @param $key
   *
   * @return static
   */
  public function incrementCounter($key) {
    if (!isset($this->counter[$key])) {
      $this->counter[$key] = 0;
    }

    $this->counter[$key]++;

    return $this;
  }

}
