<?php

namespace Drupal\entity_template;

use Drupal\Core\ParamConverter\ParamConverterInterface;
use Symfony\Component\Routing\Route;

class BlueprintTempstoreParamConverter implements ParamConverterInterface {

  /**
   * @var \Drupal\entity_template\BlueprintTempstoreRepository
   */
  protected $blueprintTempstoreRepository;

  /**
   * @var \Drupal\entity_template\TemplateBlueprintProviderManager
   */
  protected $blueprintProviderManager;

  /**
   * BlueprintTempstoreParamConverter constructor.
   *
   * @param \Drupal\entity_template\BlueprintTempstoreRepository $blueprint_tempstore_repository
   * @param \Drupal\entity_template\TemplateBlueprintProviderManager $blueprint_provider_manager
   */
  public function __construct(
    BlueprintTempstoreRepository $blueprint_tempstore_repository,
    TemplateBlueprintProviderManager $blueprint_provider_manager
  ) {
    $this->blueprintTempstoreRepository = $blueprint_tempstore_repository;
    $this->blueprintProviderManager = $blueprint_provider_manager;
  }

  /**
   * Converts path variables to their corresponding objects.
   *
   * @param mixed $value
   *   The raw value.
   * @param mixed $definition
   *   The parameter definition provided in the route options.
   * @param string $name
   *   The name of the parameter.
   * @param array $defaults
   *   The route defaults array.
   *
   * @return \Drupal\entity_template\BlueprintStorageInterface|null
   *   The converted parameter value.
   */
  public function convert($value, $definition, $name, array $defaults) {
    if (
      !isset($defaults['template_blueprint_provider']) ||
      !$this->blueprintProviderManager->hasDefinition($defaults['template_blueprint_provider'])
    ) {
      return NULL;
    }

    /** @var \Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface $provider */
    $provider = $this->blueprintProviderManager
      ->createInstance($defaults['template_blueprint_provider']);

    if ($blueprint_storage = $provider->getBlueprintStorage($defaults['blueprint_storage'])) {
      return $this->blueprintTempstoreRepository->get($blueprint_storage);
    }

    return NULL;
  }

  /**
   * Determines if the converter applies to a specific route and variable.
   *
   * @param mixed $definition
   *   The parameter definition provided in the route options.
   * @param string $name
   *   The name of the parameter.
   * @param \Symfony\Component\Routing\Route $route
   *   The route to consider attaching to.
   *
   * @return bool
   *   TRUE if the converter applies to the passed route and parameter, FALSE
   *   otherwise.
   */
  public function applies($definition, $name, Route $route) {
    return !empty($definition['blueprint_tempstore']);
  }
}
