<?php

namespace Drupal\entity_template;

use Drupal\Core\TempStore\SharedTempStoreFactory;

class BlueprintTempstoreRepository {

  /**
   * The shared tempstore factory.
   *
   * @var \Drupal\Core\TempStore\SharedTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * BlueprintTempstoreRepository constructor.
   *
   * @param \Drupal\Core\TempStore\SharedTempStoreFactory $temp_store_factory
   *   The shared tempstore factory.
   */
  public function __construct(SharedTempStoreFactory $temp_store_factory) {
    $this->tempStoreFactory = $temp_store_factory;
  }

  /**
   * Get the blueprint storaga from the tempstore if applicable.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   *
   * @return \Drupal\entity_template\BlueprintStorageInterface
   */
  public function get(BlueprintStorageInterface $blueprint_storage) {
    $key = $this->getKey($blueprint_storage);
    $tempstore = $this->getTempstore($blueprint_storage)->get($key);
    if (!empty($tempstore['blueprint_storage'])) {
      $blueprint_storage = $tempstore['blueprint_storage'];
    }
    return $blueprint_storage;
  }

  /**
   * Check whether the blueprint storage is in the tempstore.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   */
  public function has(BlueprintStorageInterface $blueprint_storage) {
    $key = $this->getKey($blueprint_storage);
    $tempstore = $this->getTempstore($blueprint_storage)->get($key);
    return !empty($tempstore['blueprint_storage']);
  }

  /**
   * Set the blueprint storage in the tempstore.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function set(BlueprintStorageInterface $blueprint_storage) {
    $key = $this->getKey($blueprint_storage);
    $this->getTempstore($blueprint_storage)->set(
      $key,
      ['blueprint_storage' => $blueprint_storage]
    );
  }

  /**
   * Delete a blueprint storage from the tempstore.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   */
  public function delete(BlueprintStorageInterface $blueprint_storage) {
    $key = $this->getKey($blueprint_storage);
    $this->getTempstore($blueprint_storage)->delete($key);
  }

  /**
   * Get the key for the tempstore.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   *
   * @return string
   */
  protected function getKey(BlueprintStorageInterface $blueprint_storage) {
    return $blueprint_storage->getTempstoreKey();
  }

  /**
   * Get the right tempstore.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   *
   * @return \Drupal\Core\TempStore\SharedTempStore
   */
  protected function getTempstore(BlueprintStorageInterface $blueprint_storage) {
    $collection = 'entity_template.blueprint_storage.'.$blueprint_storage
        ->getProvider()
        ->getPluginId();
    return $this->tempStoreFactory->get($collection);
  }
}
