<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface;
use Drupal\entity_template\BlueprintStorageInterface;

interface BlueprintProviderInterface extends PluginInspectionInterface {

  /**
   * Get all the available blueprints for a given builder
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface $builder
   *
   * @return \Drupal\entity_template\BlueprintInterface[]
   */
  public function getAllBlueprints(BuilderInterface $builder);

  /**
   * Get the available blueprints for a given builder, limiting based on
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface $builder
   * @param array $parameters
   * @param \Drupal\Core\Session\AccountInterface|NULL $account
   *
   * @return \Drupal\entity_template\BlueprintInterface[]
   */
  public function getAvailableBlueprints(BuilderInterface $builder, $parameters = [], AccountInterface $account = NULL);

  /**
   * Get a blueprint storage.
   *
   * @param $id
   *
   * @return \Drupal\entity_template\BlueprintStorageInterface
   */
  public function getBlueprintStorage($id);

  /**
   * Get the key of a given blueprint to identify it in the provider.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprintStorage
   *
   * @return string
   */
  public function getBlueprintKey(BlueprintStorageInterface $blueprint_storage);

  /**
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   *
   * @return \Drupal\Core\Url
   */
  public function getBlueprintEditUrl(BlueprintStorageInterface $blueprint_storage);

  /**
   * Get the edit template url for the blueprint.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   * @param string $key
   *
   * @return \Drupal\Core\Url
   */
  public function getBlueprintEditTemplateUrl(BlueprintStorageInterface $blueprint_storage, string $key): Url;
}
