<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Component;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Field\WidgetPluginManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;
use Drupal\entity_template\TemplateResult;
use Drupal\typed_data\PlaceholderResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class StringFieldWidgetInputComponent
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\Component
 */
class StringFieldWidgetInputComponent extends FieldWidgetInputComponent implements TemplateContextAwareComponentInterface {
  use TemplateContextAwareComponentTrait;

  /**
   * @var \Drupal\typed_data\PlaceholderResolverInterface
   */
  protected $placeholderResolver;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $template;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('plugin.manager.field.widget'),
      $container->get('typed_data.placeholder_resolver')
    );
  }

  /**
   * StringFieldWidgetInputComponent constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_manager
   * @param \Drupal\Core\Field\WidgetPluginManager $widget_manager
   * @param \Drupal\typed_data\PlaceholderResolverInterface $placeholder_resolver
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    FieldTypePluginManagerInterface $field_type_manager,
    WidgetPluginManager $widget_manager,
    PlaceholderResolverInterface $placeholder_resolver
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $entity_field_manager, $field_type_manager, $widget_manager);

    $this->placeholderResolver = $placeholder_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // @todo: Show a list of available token replacements.

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function apply(EntityInterface $entity, TemplateResult $result) {
    $value = $this->getValue();

    $data = [];
    foreach ($this->getTemplateContexts() as $name => $context) {
      $data[$name] = $context->getContextData();
    }

    foreach ($value as $delta => $item) {
      $value[$delta]['value'] = $this->placeholderResolver->replacePlaceHolders(
        $item['value'],
        $data,
        new BubbleableMetadata()
      );
    }

    $entity->{$this->getFieldName()} = $value;
  }
}
