<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Template;

use Drupal\entity_template\TemplateLazyComponentsCollection;

trait TemplateComponentTrait {

  /**
   * @var \Drupal\entity_template\TemplateComponentManager;
   */
  protected $componentManager;

  /**
   * @var \Drupal\entity_template\TemplateLazyComponentsCollection
   */
  protected $componentPluginCollection;

  /**
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidGenerator;

  /**
   * @var array
   */
  protected $configuration;

  /**
   * @return \Drupal\Component\Uuid\UuidInterface
   */
  protected function uuidGenerator() {
    if (!$this->uuidGenerator) {
      $this->uuidGenerator = \Drupal::service('uuid');
    }
    return $this->uuidGenerator;
  }

  /**
   * @return \Drupal\entity_template\TemplateComponentManager
   */
  protected function componentManager() {
    if (!$this->componentManager) {
      $this->componentManager = \Drupal::service('plugin.manager.entity_template.component');
    }

    return $this->componentManager;
  }

  /**
   * Get the template.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected function template() {
    return $this;
  }

  /**
   * Get the components of the template.
   *
   * @return \Drupal\Component\Plugin\LazyPluginCollection
   */
  public function getComponents() {
    if (!$this->componentPluginCollection) {
      $this->componentPluginCollection = new TemplateLazyComponentsCollection(
        $this->template(),
        $this->componentManager(),
        isset($this->configuration['components']) ? $this->configuration['components'] : []
      );
    }

    return $this->componentPluginCollection;
  }

  /**
   * {@inheritdoc}
   */
  public function addComponent(array $configuration) {
    $configuration['uuid'] = $this->uuidGenerator()->generate();
    $this->getComponents()->addInstanceId($configuration['uuid'], $configuration);

    return $configuration['uuid'];
  }

  /**
   * {@inheritdoc}
   */
  public function removeComponent($id) {
    $this->getComponents()->removeInstanceId($id);
  }

  /**
   * {@inheritdoc}
   */
  public function getComponent($id) {
    return $this->getComponents()->get($id);
  }
}
