<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Template;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\entity_template\BlueprintResult;

interface TemplateInterface extends PluginInspectionInterface, ConfigurableInterface {

  /**
   * The key of this template within the blueprint.
   *
   * @return string
   */
  public function getKey();

  /**
   * Whether the template always returns one entity.
   *
   * @return bool
   */
  public function isSingle();

  /**
   * Find out whether this template applies in a given blueprint.
   *
   * @return boolean
   */
  public function applies();

  /**
   * Execute the template.
   *
   * @return \Drupal\entity_template\TemplateResult
   */
  public function execute();

  /**
   * Get the components of the template.
   *
   * @return \Drupal\Component\Plugin\LazyPluginCollection
   */
  public function getComponents();

  /**
   * Add a component
   *
   * @param array $configuration
   *
   * @return string
   */
  public function addComponent(array $configuration);

  /**
   * Remove a given component.
   *
   * @param $id
   */
  public function removeComponent($id);

  /**
   * Get a component by id.
   *
   * @param $id
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface
   */
  public function getComponent($id);

  /**
   * Get the return type of this template.
   *
   * @return string
   */
  public function getReturnType();

  /**
   * Get the return type label.
   *
   * @return \Drupal\Component\Render\MarkupInterface|string
   */
  public function getReturnTypeLabel();

  /**
   * Get the label of this template.
   *
   * @return string
   */
  public function label();

  /**
   * Get all the template contexts.
   *
   * @param bool $sample
   *
   * @return \Drupal\Core\Plugin\Context\ContextInterface[]
   */
  public function getTemplateContexts($sample = FALSE);
}
