<?php

namespace Drupal\entity_template;

use Drupal\ck_api\Data\Instance\Instance;
use Drupal\ck_api\Data\TypedData\InstanceSchemaDataDefinition;
use Drupal\ck_api\Plugin\DataType\InstanceAdaptorInterface;
use Drupal\Component\Utility\Random;
use Drupal\Core\Entity\ContentEntityStorageInterface;
use Drupal\Core\Entity\TypedData\EntityDataDefinitionInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\TypedData\FieldItemDataDefinitionInterface;
use Drupal\Core\Plugin\Context\ContextDefinitionInterface;

trait SampleContextValueTrait {

  /**
   * Get the entity type manager
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected function entityTypeManager() {
    if (!$this->entityTypeManager) {
      $this->entityTypeManager = \Drupal::entityTypeManager();
    }

    return $this->entityTypeManager;
  }

  /**
   * Get the entity type bundle infor manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected function entityTypeBundleInfo() {
    if (empty($this->entityTypeBundleInfo)) {
      $this->entityTypeBundleInfo = \Drupal::service('entity_type.bundle.info');
    }

    return $this->entityTypeBundleInfo;
  }

  /**
   * Get the field type manager.
   *
   * @return \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected function fieldTypeManager() {
    if (!$this->fieldTypePluginManager) {
      $this->fieldTypePluginManager = \Drupal::service('plugin.manager.field.field_type');
    }

    return $this->fieldTypePluginManager;
  }

  /**
   * @param \Drupal\rules\Context\ContextDefinitionInterface $definition
   *
   * @return mixed
   */
  protected function generateSampleValue(ContextDefinitionInterface $definition) {
    $data_definition = $definition->getDataDefinition();
    if ($data_definition instanceof EntityDataDefinitionInterface) {
      $entity_type = $this->entityTypeManager()->getDefinition($data_definition->getEntityTypeId());
      $bundle = $entity_type->id();
      if ($entity_type->hasKey('bundle')) {
        if ($bundles = $data_definition->getBundles()) {
          $bundle = reset($bundles);
        }
        else {
          $bundles = $this->entityTypeBundleInfo()->getBundleInfo($entity_type->id());
          $bundle = key($bundles);
        }
      }

      $storage = $this->entityTypeManager()->getStorage($entity_type->id());
      return $storage instanceof ContentEntityStorageInterface ?
        $storage->createWithSampleValues($bundle) :
        $storage->create([$entity_type->getKey('bundle') => $bundle]);
    }
    else if ($data_definition instanceof FieldItemDataDefinitionInterface) {
      $class = $this->fieldTypeManager()
        ->getPluginClass($data_definition->getFieldDefinition()->getType());

      if (is_subclass_of($class, FieldItemInterface::class)) {
        return $class::generateSampleValue($data_definition->getFieldDefinition());
      }
    }
    // @todo: Seperate value generation into some kind of plugin system so that
    //        we don't have a reference to custom code here.
    else if (\Drupal::moduleHandler()->moduleExists('ck_api') && ($data_definition instanceof InstanceSchemaDataDefinition)) {
      return Instance::createInstance($data_definition->getSchema());
    }
    else if ($data_definition->getDataType() === 'string') {
      return (new Random())->string();
    }
    else if ($data_definition->getDataType() === 'integer') {
      return 4;
    }

    return NULL;
  }

}
