<?php

namespace Drupal\entity_template;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Entity\EntityInterface;

class TemplateResult {

  /**
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $entity;

  /**
   * @var string[]|\Drupal\Component\Render\MarkupInterface[]
   */
  protected $messages = [];

  /**
   * @return \Drupal\Core\Entity\EntityInterface
   */
  public function getEntity(): ?EntityInterface {
    return $this->entity;
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return static
   */
  public function setEntity(EntityInterface $entity) {
    $this->entity = $entity;

    return $this;
  }

  /**
   * Get all messages.
   *
   * @return \Drupal\Component\Render\MarkupInterface[]|string[]
   */
  public function getMessages() {
    return $this->messages;
  }

  /**
   * Add a message
   *
   * @param string|\Drupal\Component\Render\MarkupInterface $message
   *
   * @return static
   */
  public function addMessage($message) {
    if (!is_string($message) && !($message instanceof MarkupInterface)) {
      throw new \InvalidArgumentException('Message must be a string or implement Markup Interface');
    }

    $this->messages[] = $message;

    return $this;
  }

}
