<?php
/**
 * Created by PhpStorm.
 * User: Rob
 * Date: 21/04/2020
 * Time: 15:50
 */

namespace Drupal\entity_template;

use Drupal\Core\StringTranslation\TranslatableMarkup;

trait TypeLabelTrait {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * Get the entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected function entityTypeManager() {
    return $this->entityTypeManager;
  }

  /**
   * Get the entity type bundle info.
   *
   * @return \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected function entityTypeBundleInfo() {
    return $this->entityTypeBundleInfo;
  }

  /**
   * Get the label of a type.
   *
   * @param string $type
   */
  protected function getTypeLabel(string $type) {
    list(,$entity_type_id, $bundle) = explode(':', $type.':');

    $entity_type_label = $this->entityTypeManager()->getDefinition($entity_type_id)->getLabel();
    if (!$bundle || !$this->entityTypeManager()->getDefinition($entity_type_id)->hasKey('bundle')) {
      return $entity_type_label;
    }

    $bundle_label = $this->entityTypeBundleInfo()->getBundleInfo($entity_type_id)[$bundle]['label'];

    return new TranslatableMarkup('@bundle @entity_type', [
      '@bundle' => $bundle_label,
      '@entity_type' => $entity_type_label,
    ]);
  }

}
